/*
 Java version of Brill's Part-of-Speech Tagger
 Copyright (C) 2003-2004, Jimmy Lin

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, please visit 
 http://www.gnu.org/copyleft/gpl.html

 */
package edu.mit.csail.brill;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;

/**
 * The Java interface to Eric Brill's part-of-speech tagger. It is written as a
 * wrapper around {@link BTagger}, the raw JNI interface around the original
 * tagger.
 * 
 * <p>
 * Here is the complete tagset:
 * <ol>
 * <li><b><code>CC</code> </b> Coordinating conjunction
 * <li><b><code>CD</code> </b> Cardinal number
 * <li><b><code>DT</code> </b> Determiner
 * <li><b><code>EX</code> </b> Existential there
 * <li><b><code>FW</code> </b> Foreign word
 * <li><b><code>IN</code> </b> Preposition/subord. conjunction
 * <li><b><code>JJ</code> </b> Adjective
 * <li><b><code>JJR</code> </b> Adjective, comparative
 * <li><b><code>JJS</code> </b> Adjective, superlative
 * <li><b><code>LS</code> </b> List item marker
 * <li><b><code>MD</code> </b> Modal
 * <li><b><code>NN</code> </b> Noun, singular or mass
 * <li><b><code>NNS</code> </b> Noun, plural
 * <li><b><code>NNP</code> </b> Proper noun, singular
 * <li><b><code>NNPS</code> </b> Proper noun, plural
 * <li><b><code>PDT</code> </b> Predeterminer
 * <li><b><code>POS</code> </b> Possessive ending
 * <li><b><code>PRP</code> </b> Personal pronoun
 * <li><b><code>PRP$</code> </b> Possessive pronoun
 * <li><b><code>RB</code> </b> Adverb
 * <li><b><code>RBR</code> </b> Adverb, comparative
 * <li><b><code>RBS</code> </b> Adverb, superlative
 * <li><b><code>RP</code> </b> Particle
 * <li><b><code>SYM</code> </b> Symbol (mathematical or scientific)
 * <li><b><code>TO</code> </b> to
 * <li><b><code>UH</code> </b> Interjection
 * <li><b><code>VB</code> </b> Verb, base form
 * <li><b><code>VBD</code> </b> Verb, past tense
 * <li><b><code>VBG</code> </b> Verb, gerund/present participle
 * <li><b><code>VBN</code> </b> Verb, past participle
 * <li><b><code>VBP</code> </b> Verb, non-3rd ps. sing. present
 * <li><b><code>VBZ</code> </b> Verb, 3rd ps. sing. present
 * <li><b><code>WDT</code> </b> wh-determiner
 * <li><b><code>WP</code> </b> wh-pronoun
 * <li><b><code>WP$</code> </b> Possessive wh-pronoun
 * <li><b><code>WRB</code> </b> wh-adverb
 * <li><b><code>#</code> </b> Pound sign
 * <li><b><code>$</code> </b> Dollar sign
 * <li><b><code>.</code> </b> Sentence-final punctuation
 * <li><b><code>,</code> </b> Comma
 * <li><b><code>:</code> </b> Colon, semi-colon
 * <li><b><code>(</code> </b> Left bracket character
 * <li><b><code>)</code> </b> Right bracket character
 * <li><b><code>"</code> </b> Straight double quote
 * <li><b><code>`</code> </b> Left open single quote
 * <li><b><code>"</code> </b> Left open double quote
 * <li><b><code>'</code> </b> Right close single quote
 * <li><b><code>"</code> </b> Right close double quote
 * </ol>
 * 
 * @author Jimmy Lin
 */

public class BrillTagger {
	private static org.apache.log4j.Logger sLogger = org.apache.log4j.Logger
			.getLogger(BrillTagger.class);

	static {
		System.loadLibrary("BTagger");
	}

	private BTagger posTagger;

	/**
	 * Creates an instance of Brill's part-of-speech tagger.
	 */
	public BrillTagger() {
		posTagger = new BTagger();
	}

	/**
	 * Initializes the tagger.
	 */
	public void setup() {
		posTagger.setup();
	}

	/**
	 * Destroys the tagger.
	 */
	public void destroy() {
		posTagger.destroy();
	}

	/**
	 * Part-of-speech tags a sentence.
	 * 
	 * @param sentence
	 *            the sentence to tag
	 * @return TaggedToken[] an array containing the tag output
	 */
	public Sentence tag(String sentence) {
		sLogger.debug("\nRaw tag input: " + sentence);
		String tokenized = tokenize(sentence);
		sLogger.debug("After tokenization: " + tokenized);
		String tagged = posTagger.tag(tokenized);
		sLogger.debug("Raw tag output: " + tagged);

		String[] tokens = tagged.split(" ");
		TaggedToken[] taggedtokens = new TaggedToken[tokens.length];

		for (int i = 0; i < tokens.length; i++) {
			String cur = tokens[i];
			String p[] = cur.split("/");

			taggedtokens[i] = new TaggedToken(p[0], p[1]);
		}

		return new Sentence(sentence, tokenized, taggedtokens);
	}

	public static String tokenize(String s) {
		return PennTokenizer.tokenize(s);
	}

	public static void main(String arg[]) throws IOException {
		BufferedReader stdin = new BufferedReader(new InputStreamReader(
				System.in));
		BrillTagger tagger = new BrillTagger();

		tagger.setup();

		while (true) {
			System.out.println("Enter sentence to tag:");
			String sentence = stdin.readLine();
			if (sentence.equals("exit"))
				break;

			Sentence result = tagger.tag(sentence);
			System.out.println(result.getOriginalSentence());
			System.out.println(result.getTokenizedSentence());
			System.out.println(result.toString());
		}

		tagger.destroy();
	}
}