/*
Java version of Brill's Part-of-Speech Tagger
Copyright (C) 2003-2004, Jimmy Lin

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, please visit 
http://www.gnu.org/copyleft/gpl.html

*/
package edu.mit.csail.brill;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

/**
 * Object representing a tagged sentence. It is essentially a collection of
 * {@link edu.mit.csail.brill.TaggedToken}s.
 * 
 * @author Jimmy Lin
 */
public class Sentence {
	protected static org.apache.log4j.Logger sLogger = org.apache.log4j.Logger
			.getLogger(Sentence.class);

	protected List mTaggedTokens;

	protected String mOriginalSentence;

	protected String mTokenizedSentence;

	/**
	 * Creates a sentence object.
	 * 
	 * @param original original untokenized sentence
	 * @param tokenized tokenized sentence
	 * @param tokens array of tokens
	 */
	public Sentence(String original, String tokenized, TaggedToken[] tokens) {
		mOriginalSentence = original;
		mTokenizedSentence = tokenized;

		mTaggedTokens = new ArrayList();

		for (int i = 0; i < tokens.length; i++) {
			addToken(tokens[i]);
		}

		initOffsets();
	}

	/**
	 * Creates a sentence object.
	 * 
	 * @param original original untokenized sentence
	 * @param tokenized tokenized sentence
	 * @param tokens array of (tokens, tag) pairs
	 */
	public Sentence(String original, String tokenized, String[][] tokens) {
		mOriginalSentence = original;
		mTokenizedSentence = tokenized;

		mTaggedTokens = new ArrayList();

		for (int i = 0; i < tokens.length; i++) {
			addToken(new TaggedToken(tokens[i][0], tokens[i][1]));
		}

		initOffsets();
	}

	private void addToken(TaggedToken token) {
		mTaggedTokens.add(token);
		token.setSentence(this);
	}

	/**
	 * Returns the original untokenized sentence.
	 */
	public String getOriginalSentence() {
		return mOriginalSentence;
	}

	/**
	 * Returns the tokenized sentence.
	 */
	public String getTokenizedSentence() {
		return mTokenizedSentence;
	}

	/**
	 * Returns an interator over the {@link edu.mit.csail.brill.TaggedToken}s.
	 */
	public Iterator getTokens() {
		return Collections.unmodifiableList(mTaggedTokens).iterator();
	}

	/**
	 * Returns the character offset of a token.
	 */
	public int getTokenOffset(TaggedToken tok) {
		return tok.getOffset();
	}

	/**
	 * Returns the character offset of the <i>i</i>th token.
	 */
	public int getTokenOffset(int i) {
		return getTokenOffset(getToken(i));
	}

	/**
	 * Returns the <i>i</i>th token.
	 */
	public TaggedToken getToken(int i) {
		return (TaggedToken) mTaggedTokens.get(i);
	}

	/**
	 * Returns the number of tokens.
	 */
	public int numTokens() {
		return mTaggedTokens.size();
	}

	public String toString() {
		StringBuffer sb = new StringBuffer();

		for (int i = 0; i < numTokens(); i++) {
			TaggedToken cur = getToken(i);

			sb.append(cur.getToken());
			sb.append("/");
			sb.append(cur.getTokenType());

			if (i != numTokens())
				sb.append(" ");
		}

		return sb.toString();
	}

	protected void initOffsets() {
		StringBuffer currentString = new StringBuffer(mOriginalSentence);
		int offset = 0;
		for (Iterator i = mTaggedTokens.iterator(); i.hasNext();) {
			//sLogger.debug("currentString: " + currentString);
			TaggedToken token = (TaggedToken) i.next();
			//sLogger.debug("currentToken: " + token);

			int tokIndex = currentString.toString().indexOf(token.getToken());
			if (tokIndex == -1) {
				throw new RuntimeException(
						"raw sentence didn't contain token! rawSentence: "
								+ mOriginalSentence + "currentString: "
								+ currentString + " token: " + token);
			}

			offset = offset + tokIndex;
			token.setOffest(offset);
			offset = offset + token.getToken().length();
			currentString.delete(0, tokIndex + token.getToken().length());

		}
	}

	public int hashCode() {
		return mOriginalSentence.hashCode();
	}

	public boolean equals(Object obj) {
		return equals((Sentence) obj);
	}

	public boolean equals(Sentence s) {
		if (!mOriginalSentence.equals(s.getOriginalSentence()))
			return false;

		if (!mTokenizedSentence.equals(s.getTokenizedSentence()))
			return false;

		if (numTokens() != s.numTokens())
			return false;

		for (int i = 0; i < numTokens(); i++) {
			TaggedToken cur = getToken(i);

			if (!cur.getToken().equals(s.getToken(i).getToken()))
				return false;

			if (!cur.getTokenType().equals(s.getToken(i).getTokenType()))
				return false;
		}

		return true;
	}
}