/*
Java version of Brill's Part-of-Speech Tagger
Copyright (C) 2003-2004, Jimmy Lin

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, please visit 
http://www.gnu.org/copyleft/gpl.html

*/
package edu.mit.csail.brill;

/**
 * Object representing a tagged token. A {@link edu.mit.csail.brill.Sentence }
 * is a collection of such objects.
 * 
 * @author Jimmy Lin
 */
public class TaggedToken {
	private String mToken;

	private String mTokenType;

	private Sentence mSentence = null;

	private int mOffset = -1;

	/**
	 * Constructs a tagged token.
	 * 
	 * @param s
	 *            the token
	 * @param pos
	 *            the part-of-speech tag
	 */
	public TaggedToken(String s, String pos) {
		mToken = s;
		mTokenType = pos;
	}

	/**
	 * Constructs a tagged token.
	 * 
	 * @param s
	 *            the token
	 * @param pos
	 *            the part-of-speech tag
	 * @param sent
	 *            the sentence to which this token belongs
	 * @param i
	 *            the offset of this token relative to the enclosing sentence
	 */
	public TaggedToken(String s, String pos, Sentence sent, int i) {
		this(s, pos);
		mSentence = sent;
		mOffset = i;
	}

	/**
	 * Returns the character offset of this token with respect to the enclosing
	 * sentence.
	 */
	public int getOffset() {
		return mOffset;
	}

	/**
	 * Returns the sentence to which this token belongs.
	 */
	public Sentence getSentence() {
		return mSentence;
	}

	/**
	 * Returns the token string.
	 */
	public String getToken() {
		return mToken;
	}

	/**
	 * Returns the part-of-speech type of this token.
	 */
	public String getTokenType() {
		return mTokenType;
	}

	/**
	 * Sets the character offset of this token with respect to the enclosing
	 * sentence.
	 */
	public void setOffest(int i) {
		mOffset = i;
	}

	/**
	 * Sets the sentence to which this token belongs.
	 */
	public void setSentence(Sentence s) {
		mSentence = s;
	}

	public String toString() {
		StringBuffer result = new StringBuffer(TaggedToken.class.toString());
		result.append(" [tokenString=");
		result.append(mToken);
		result.append(",tokenType=");
		result.append(mTokenType);
		result.append("]");
		return result.toString();
	}

	public int hashCode() {
		return mToken.hashCode() + mTokenType.hashCode();
	}

	public boolean equals(Object obj) {
		return equals((TaggedToken) obj);
	}

	public boolean equals(TaggedToken tok) {
		if (!mToken.equals(tok.getToken()))
			return false;

		if (!mTokenType.equals(tok.getTokenType()))
			return false;

		Sentence sent = tok.getSentence();
		if (mSentence == null && sent == null)
			return true;

		if (mSentence != sent)
			return false;

		if (mOffset != tok.getOffset())
			return false;

		return true;
	}
}